;;; metaspec-pkg.el --- package descriptor -*- no-byte-compile: t; lexical-binding: t -*-
;;; Generated by metaspectre; edit src/elpa.lisp instead.

(define-package "metaspec" "1.0.9"
  "Metaspec render of the dpANS3R Common Lisp standard"
  '((emacs "25.1"))
  :url "https://metaspec.dev/"
  :keywords '("docs" "languages" "lisp"))

;;; metaspec-pkg.el ends here
